//********************************************************************
//*** ResourcePool/SQL/SQLFetcher.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLFetcher.hh,v 1.4 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_SQLFETCHER_HH
#define RESOURCEPOOL_SQL_SQLFETCHER_HH


namespace fatalmind {
namespace SQL {

/**
 * This class allows fetching of data (rows) from
 * \link fatalmind::SQL::RTSQLSelect SQLSelect \endlink or 
 * \link fatalmind::SQL::RTSQLSelectRow SQLSelectRow \endlink 
 * commands.
 * 
 * To fetch data you must derive a "Fetcher" from this class and pass it to 
 * \link fatalmind::SQL::RTSQLSelect SQLSelect \endlink or 
 * \link fatalmind::SQL::RTSQLSelectRow SQLSelectRow \endlink 
 * constructor.
 * For each retrieved row the bound output parameters are set and the process() method is invoked. Implementations
 * can stop the retrieval by returning "false" from their process() implementation.
 *
 * Optionally you can also overwrite the done() method which is called when the retrieval has finished.
 *
 * The usage of a Fetcher is the only way to retrieve multiple rows and it is therefore an mandatory parameter
 * to all \link fatalmind::SQL::RTSQLSelect SQLSelect \endlink  constructors. 
 * For single row fetching via \link fatalmind::SQL::RTSQLSelectRow SQLSelectRow \endlink  a fetcher class is an 
 * optional parameter,
 * alternativley you can just access the output parameters after completion of the 
 * \link fatalmind::SQL::RTSQLSelectRow SQLSelectRow \endlink command.
 * \since 0.99.22
 */
class SQLFetcher {
	public:
		virtual ~SQLFetcher() {
		}

		/**
		 * This method is to be overwritten by the user and
		 * will be invoked for each row.
		 * The variables you have bound are filled accordingly.
		 * 
		 * As implementor of this method you may chose to stop
		 * fetching of more data. You can do so by returning 
		 * false. 
		 *
		 * @param[in] row The rownumber to be processed. Starting at 0.
		 *
		 * \return true - to proceed fetching of further records
		 * \return false - to stop fetching and terminate the \link fatalmind::SQL::RTSQLSelect SQLSelect \endlink execution
		 */
		virtual bool process(int row) = 0;

		/**
		 * This method is called if all records were fetched succesfully.
		 * This method will NOT be called if execution was aborted because of an
		 * exception.
		 * 
		 * The default implementation does nothing.
		 *
		 * @param[in] more     Indicates if all data was fetched (false) or
		 *                     if (eventually) more data could have been fetched (true). 
		 * @param[in] fetched  The number of records fetched
		 */

		virtual void done(bool more, int fetched) {
			// The default implementation does nothing
		}
};

class NopSQLFetcher: public SQL::SQLFetcher {
	public:
		NopSQLFetcher() {
		};
		virtual ~NopSQLFetcher() {
		}
        virtual bool process(int row) {
			return true;
		};
};

extern NopSQLFetcher NopFetcher;

} //namespace SQL
} //namespace fatalmind

#endif
